#!/usr/bin/perl
#
# Firewall Tester client v.0.1
# by Andrea Barisani (lcars@infis.univ.trieste.it) 

require 'getopts.pl';
use Net::RawIP;
Getopts('f:d:');

die "Usage $0 -f <conf_file> -d <delay, 0.25 = 250 ms>" unless ($opt_f & $opt_d);

my $delay = $opt_d;
my $tcp_payload = 'ftestertcpprobe';
my $udp_payload = 'ftesterudpprobe';
my $icmp_payload = 'ftestericmpprobe';
my $stopsignal = 'ftesterstop';

open (OUT,">>./ftest.log") || die "cannot create log file";
print OUT ("# Ftest started on ".`date`);

open (CONF,$opt_f) or die "Can't open $opt_f";
while (<CONF>) {
	next if  /^#|^\s/; 
	my @fields = split /:/;
	die unless @fields >= 6;
	
	$psh = 0; $ack = 0; $rst = 0; $syn = 0; $urg = 0; $fin = 0;
	
	$psh = 1 if $fields[4] =~ /P/;
	$ack = 1 if $fields[4] =~ /A/;
	$rst = 1 if $fields[4] =~ /R/;
	$syn = 1 if $fields[4] =~ /S/;
	$urg = 1 if $fields[4] =~ /U/;
	$fin = 1 if $fields[4] =~ /F/;
		
	if ($_ !~ /-|stop_signal/) {
		if ($fields[5] =~ TCP) {
		send_tcp($fields[0],$fields[1],$fields[2],$fields[3],$fields[4],$tcp_payload);
		}
		if ($fields[5] =~ UDP) {
		send_udp($fields[0],$fields[1],$fields[2],$fields[3],$fields[4],$udp_payload);
		}
		if ($fields[5] =~ ICMP) {
		send_icmp($fields[0],$fields[1],$fields[2],$fields[3],$fields[4],$icmp_payload,$fields[6],$fields[7]);
		}
	} else {

		if ($fields[0] =~ /[0-9]+\-[0-9]+/ ) {
			my @ip_range = split (/\.|-/,$fields[0]);
		
			for my $i ( $ip_range[3] .. $ip_range[4] ) {
				$source_addr = $ip_range[0].'.'.$ip_range[1].'.'.$ip_range[2].'.'.$i;
			
				if ($fields[1] =~ /[0-9]+\-[0-9]+/ ) {
					@src_port_range = split (/-/,$fields[1]);
					for my $l ( $src_port_range[0] .. $src_port_range[1] ) {
					
					if ($fields[3] =~ /[0-9]+\-[0-9]+/ ) {
						@dst_port_range = split (/-/,$fields[3]);
						for my $h ( $dst_port_range[0] .. $dst_port_range[1] ) {
							
							if ($fields[5] =~ TCP) {
							send_tcp($source_addr,$l,$fields[2],$h,$fields[4],$tcp_payload);
							}
							if ($fields[5] =~ UDP) {
							send_udp($source_addr,$l,$fields[2],$h,$fields[4],$udp_payload);
							}
							if ($fields[5] =~ ICMP) {
							send_icmp($source_addr,$l,$fields[2],$h,$fields[4],$icmp_payload,$fields[6],$fields[7]);
							}
						}
							
					} else { die "/* one or more entries in ftest.conf are incorrect! */" }
					}
			
				} else { die "/* one or more entries in ftest.conf are incorrect! */" }	
			} 
		
		} else {
		
			if ($fields[0] =~ /stop_signal/ ) {
				my @stop_fields = split /=|:/;
			
				if ($stop_fields[6] =~ TCP) {
	   			 	send_tcp($stop_fields[1],$stop_fields[2],$stop_fields[3],$stop_fields[4],$stop_fields[5],$stopsignal);
					print OUT ("# Ftest stopped on ".`date`);
					close (CONF);
				}
				if ($stop_fields[6] =~ UDP) {
	   			 	send_udp($stop_fields[1],$stop_fields[2],$stop_fields[3],$stop_fields[4],$stop_fields[5],$stopsignal);
					print OUT ("# Ftest stopped on ".`date`);
					close (CONF);
				}
				if ($stop_fields[6] =~ ICMP) {
					send_icmp($stop_fields[1],$stop_fields[2],$stop_fields[3],$stop_fields[4],$stop_fields[5],$stopsignal,$stop_fields[6],$stop_fields[7]);
					print OUT ("# Ftest stopped on ".`date`);
					close (CONF);
				}
			} else { die  "/* one or more entries in ftest.conf are incorrect! */" }	
		}
	}
}

sub send_tcp {
	
		$a = new Net::RawIP({
		ip => {
			ttl => 200,
			id => 1,
			saddr => $_[0],
                	daddr => $_[2],
			},
		tcp => {
                	source => $_[1],
                	dest => $_[3],
			psh => $psh, 
			ack => $ack,
			rst => $rst,
			syn => $syn, 
			urg => $urg, 
			fin => $fin, 
                	data => $_[5]}
		});
		$a->send($delay,1);

		print OUT ("$_[0]:$_[1] > $_[2]:$_[3] ");
		
		$flags = 0;	
		
		if ($_[4] =~ /P/ ) {
			$flags = 8;
			}
               	if ($_[4] =~ /A/ ) {
			$flags = $flags + 16;
			}
               	if ($_[4] =~ /R/ ) {
			$flags = $flags + 4;
			}
               	if ($_[4] =~ /S/ ) {
			$flags = $flags + 2;
			}
               	if ($_[4] =~ /U/ ) {
			$flags = $flags + 32;
			}
               	if ($_[4] =~ /F/ ) {
			$flags = $flags + 1;
			}
       		print OUT ("$flags TCP\n"); 
	}

sub send_udp {
	
		$a = new Net::RawIP({
		ip => {
			ttl => 200,
			id => 1,
			saddr => $_[0],
                	daddr => $_[2],
			},
		udp => {
                	source => $_[1],
                	dest => $_[3],
                	data => $_[5]}
		});
		$a->send($delay,1);

		print OUT ("$_[0]:$_[1] > $_[2]:$_[3] UDP\n");
		
	}

sub send_icmp {
	
		$a = new Net::RawIP({
		ip => {
			ttl => 200,
			id => 1,
			saddr => $_[0],
                	daddr => $_[2],
			},
		icmp => {
                	type => $_[6],
			code => $_[7],
			data => $_[5]}
		});
		$a->send($delay,1);

		print OUT ("$_[0] > $_[2] ICMP $_[6] $_[7]");
	}
